<?php
use Drupal\user\Entity\User;
use Drupal\bibcite_entity\Entity\Reference;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\bibcite_import_orcid\Import;

  /**
 * Import Publications
 *
 */
function bibcite_prep_import_orcid_batch($work, $user, &$context) {
    $import = new Import;
    $uid = \Drupal::currentUser()->id();
    $orcid = $_SESSION['orcid'];
    $summary = $work['work-summary'][0];
    $orcid_type = $summary["type"];
    $type = $import->bibcite_entity_types($orcid_type);
    $title = substr($summary['title']['title']['value'], 0, 250);
    $publication_date = $summary['publication-date']['year']['value'];
    $work_put_code = $summary['put-code'];
    $workData = $import->getOrcidWorkData($work_put_code, $orcid);
    $description = $workData['short-description'];
    $bibtex = $workData["citation"]["citation-value"];
    !empty($bibtex) ? $bibtex_info = $import->parce_bibtex($bibtex) : $bibtex_info = "";
    $options = array();
    $contribs = $workData['contributors']['contributor'];
    // Make sure authors are available
    if (empty($contribs)) {
        // Use Authors from bibtex
        if (!empty($bibtex_info)) {
            $contribs = $bibtex_info['bibcite_authors'];
            foreach ($contribs as $contributor) {
                $contributors[] = $contributor["first"] . ' ' . $contributor["last"];
            }
        } else {
        // If no authors on bibtex, add an option based on Orcid info
        $person_data = $import->getOrcidPersonData($orcid);
        $contributors[] = $person_data['name']['given-names']['value'] . ' ' . $person_data['name']['family-name']['value'];
        }
    } else {
        $contributors = array_map(function($contrib){
            return $contrib['credit-name']['value'];
        },$contribs);
    }
    // Search for the reference in bibcite
    $query = \Drupal::entityQuery('bibcite_reference')
    ->condition('title', $title)
    ->condition('type', $type)
    ->execute();

    if (empty($query)){
    // If the publication is new
        // Add an option for each contributor
        foreach ($contributors as $contributor) {
            $options[] = $contributor;
        }
    } else {
        // If the publication already exists
        $bibcite_reference = Reference::load(reset($query));
        $pub_refs = $bibcite_reference->author->referencedEntities();
        if(!empty($pub_refs)){
            // Get the contributors name from the existing publication
            $pub_contributors = [];
            foreach ($pub_refs as $referenced_contributor){
                $key = $referenced_contributor->id();
                $pub_contributors[$key] = $referenced_contributor->getName();
            };
            // Look for linked authors
            $linked_authors = [];
            foreach ($pub_contributors as $key => $contrib_id) {
                $query_users = \Drupal::entityQuery('user')
                    ->condition('field_author', $key)
                    ->execute();
                if($query_users){
                    $linked_authors[$key] = reset($query_users);
                }
            }
        } else {
            foreach ($contributors as $contributor) {
                $options[] = $contributor;
            }
        }

        // Check if the current user is already an author
        foreach ($pub_contributors as $key => $contributor) {
            if (array_key_exists($key,$linked_authors)){
                $attributes = [];
                // Is the onwer?
                if(in_array($uid,$linked_authors)){
                    $options[$key] = $contributor;
                    $attributes[$key] = array('disabled' => 'disabled', 'readonly' => true);
                } else {
                // other linked users
                    $options[$key] = $contributor;
                    $attributes[$key] = array('disabled' => 'disabled');
                }
            } else {
                // Unlinked users
                $options[$key] = $contributor;
            }
            // Are there authors missing
            if (count($pub_contributors) < count($contributors)) {
                $missing_contribs = array_diff($contributors, $pub_contributors);
                // Add an option for each contributor
                foreach ($missing_contribs as $key => $contributor) {
                    $options[$key] = $contributor;
                }
            }
        }

        // Get the user contributor ids
        $user_authors_ids = array_column($user->field_author->getValue(),'target_id','target_id');
    }

    // Prepare output
    $reference_data = [
        'type' => $type,
        'title' => $title,
        'secundary_title' => preg_replace(array('/\s{2,}/', '/[\t\n]/'), ' ', $workData['journal-title']['value']),
        'bibcite_year' => $publication_date,
        'work_code' => $work_put_code,
        'bibcite_abst_e' => $description,
        'contribs' => $options,
        'user_authors' => $user_authors_ids,
        'contribs_atts' => $attributes,
        'bibtex' => $bibtex_info,
    ];

    $work_ext_references = $workData['external-ids']['external-id'];

    // add more references if available
    foreach ($work_ext_references as $ext_ref) {
        switch ($ext_ref['external-id-type']) {
            case 'doi':
                $reference_data['bibcite_url'] = $ext_ref['external-id-url'];
                $reference_data['bibcite_doi'] = $ext_ref['external-id-value'];
                break;
            case 'issn':
                $reference_data['bibcite_issn'] = $ext_ref['external-id-value'];
                break;
            case 'isbn':
                $reference_data['bibcite_isbn'] = $ext_ref['external-id-value'];
                break;
            default:
                $reference_data['bibcite_other_number'] = $ext_ref['external-id-value'];
                break;
        }
    }

    // Return info
    $context['results']['work'][] = json_encode($reference_data);

  }

/**
 * Complete a batch process.
 *
 * @param bool $success
 *   A boolean indicating whether the batch has completed successfully.
 * @param array $results
 *   The value set in $context['results'] by callback_batch_operation().
 * @param array|bool $operations
 *   If $success is FALSE, contains the operations that remained unprocessed.
 */
function bibcite_prep_import_orcid_batch_finished($success, $results, $operations){
    if ($success) {
        $id = time();
        $_SESSION['works_data_' . $id ] = $results['work'];
        return new RedirectResponse('/orcid-import/import/' . $id);
    }else{
        $message = t('An error occured while preparing the import from ORCID. Check the logs for more information. ') . $operations . t(' items to be processed.');
        $messenger = \Drupal::messenger();
        $messenger->addMessage($message);
    }
}
function bibcite_import_orcid_batch($work, $uid, &$context) {
    // Decode the reference work
    $work_details = json_decode($work['pub'], true);
    // Make sure a numeric
    $selected_author = $work['author'];
    $pub_authors = [];
    $import = new Import;
    if(isset($selected_author)){
        foreach ($work_details["contribs"] as $key => $value) {
            $id = $import->author_to_id($value);
            $pub_authors[] = $id;
            if ($key == $selected_author){
                $user_id = \Drupal::currentUser()->id();
                $user = User::load($user_id);
                assert($user instanceof User);
                $authors = $user->field_author->getValue();
                $current_author_ids = isset($authors) ? array_column($authors, 'target_id') : [];
                if (!in_array($id, $current_author_ids)){
                    $user->field_author[] = $id;
                    $user->save();
                }
            }
        }
    }

    $title = $work_details['title'];
    $type = $work_details['type'];

    // Check if the reference is already on bibcite
    $query_reference = \Drupal::entityQuery('bibcite_reference')
    ->condition('title', $title)
    ->condition('type', $type)
    ->execute();

    if (empty($query_reference)) {
        // create a new reference
        $bibcite_reference = Reference::create($work_details);
        $context['results']['create'][] = $title;
    } else {
        // load existing
        $bibcite_reference = Reference::load(reset($query_reference));
        $bibcite_reference->author->setValue($pub_authors);
        if (isset($work_details["bibtex"]["bibcite_pages"])){
            $bibcite_reference->bibcite_pages->setValue($work_details["bibtex"]["bibcite_pages"]);}
        if (isset($work_details["secundary_title"])){
            $bibcite_reference->bibcite_secondary_title->setValue($work_details["secundary_title"]);
        } elseif (isset($work_details["bibtex"]["bibcite_booktitle"])){
            $bibcite_reference->bibcite_secondary_title->setValue($work_details["bibtex"]["bibcite_booktitle"]);}
        if (isset($work_details["bibtex"]["bibcite_volume"])){
            $bibcite_reference->bibcite_volume->setValue($work_details["bibtex"]["bibcite_volume"]);}
        if (isset($work_details["bibtex"]["bibcite_section"])){
            $bibcite_reference->bibcite_section->setValue($work_details["bibtex"]["bibcite_section"]);}
        if (isset($work_details["bibtex"]["bibcite_publisher"])){
            $bibcite_reference->bibcite_publisher->setValue($work_details["bibtex"]["bibcite_publisher"]);}
        if (isset($work_details["bibcite_url"])){
            $bibcite_reference->bibcite_url->setValue($work_details["bibcite_url"]);}
        if (isset($work_details["bibcite_other_number"])){
            $bibcite_reference->bibcite_other_number->setValue($work_details["bibcite_other_number"]);}
        if (isset($work_details["bibcite_issn"])){
            $bibcite_reference->bibcite_issn->setValue($work_details["bibcite_issn"]);}
        if (isset($work_details["bibcite_isbn"])){
            $bibcite_reference->bibcite_isbn->setValue($work_details["bibcite_isbn"]);}
        if (isset($work_details["bibcite_doi"])){
            $bibcite_reference->bibcite_doi->setValue($work_details["bibcite_doi"]);}
        $context['results']['update'][] = $title;
    }

    // Save authors on ref
    $bibcite_reference->author->setValue($pub_authors);
    $bibcite_reference->save();
}

/**
 * Complete a batch process.
 *
 * @param bool $success
 *   A boolean indicating whether the batch has completed successfully.
 * @param array $results
 *   The value set in $context['results'] by callback_batch_operation().
 * @param array|bool $operations
 *   If $success is FALSE, contains the operations that remained unprocessed.
 */
function bibcite_import_orcid_batch_finished($success, $results, $operations) {

    // The 'success' parameter means no fatal PHP errors were detected. All
    // other error management should be handled using 'results'.
    if ($success) {
        // Cleanup duplicate authors
        $user_id = \Drupal::currentUser()->id();
        $user = User::load($user_id);
        assert($user instanceof User);
        $authors = $user->field_author->getValue();
        $authors = array_column($authors, 'target_id');
        $authors = array_unique($authors);
        $user->field_author = $authors;
        $user->save();

        $id = time();
        $_SESSION['results_orcid_' . $id ] = $results;
        return new RedirectResponse('/orcid-import/success/' . $id);
    }else{
        $message = t('An error occured while importing.');
        }
    $messenger = \Drupal::messenger();
    $messenger->addMessage($message);
}
