<?php

namespace Drupal\bibcite_import_orcid\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;


/**
 * Provides a ORCID Prepare form.
 */
class PrepareImport extends FormBase
{

    const LOG_CHANNEL = 'bibcite_import_orcid';
    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'prepare_import';
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state)
    {
        $form['submit'] = array('#type' => 'submit', '#value' => t('Import from ORCID'));
        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state)
    {
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
    // Reset variable
    $_SESSION['all_works']=[];
    // Get the user orcid id
    $uid = \Drupal::currentUser()->id();
    $user = User::load($uid);
    $orcid = trim($user->get('field_orcid')->value);
    $_SESSION['orcid'] = $orcid;
    // array for the batch
    $works = $this->getAllWorksFromOrcid($orcid);

    // Prepare form batch

    $batch = array(
      'title' => t('Fetching publications from ORCID, please wait...'),
      'operations' => [],
      'finished' => 'bibcite_prep_import_orcid_batch_finished',
      'init_message' => t('Preparing the form'),
      'progress_message' => t('@current of @total. Estimated duration: @estimate.'),
      'error_message' => t('There was an error on the import process.'),
      'file' => \Drupal::service('extension.list.module')->getPath('bibcite_import_orcid') . '/bibcite_import_orcid.batch.inc',
    );
    foreach ($works as $work) {
      $batch['operations'][] = ['bibcite_prep_import_orcid_batch', [$work, $user]];
    };
    batch_set($batch);
    }

    public function getAllWorksFromOrcid($orcid)
    {

        //Do something here to get any data.
        \Drupal::logger(self::LOG_CHANNEL)->notice("Fetching ORCID data for user: " . $orcid);
        //set url for user works
        $url = "https://pub.orcid.org/" . $orcid . "/works";
        //Initialize cURL session
        $ch = curl_init();
        //Define cURL options
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 600);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Accept: application/vnd.orcid+json',
        ]);
        //Receive the result of the cURL session
        $curl_result = curl_exec($ch);
        //Check if there were any errors during the curl session
        if (curl_error($ch)) {
            $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            \Drupal::logger(self::LOG_CHANNEL)->notice("cURL error while fetching ORCID works for user " . $orcid . ": " . curl_error($ch));
            if ($http_code == 504) {
                \Drupal::logger(self::LOG_CHANNEL)->notice("ORCID API Timeout");
            }
        }
        //Close cURL session
        curl_close($ch);

        //Decodes the information to JSON
        $all_works = json_decode($curl_result, true);

        $result = $all_works['group'];

        return $result;
    }
}
